"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var make_enricher_exports = {};
__export(make_enricher_exports, {
  Enricher: () => Enricher,
  makeEnricher: () => makeEnricher
});
module.exports = __toCommonJS(make_enricher_exports);
var import_assert = require("@code-like-a-carpenter/assert");
var import_errors = require("../../errors");
var import_handlers = require("../common/handlers");
function makeEnricher(constructor, sdk) {
  const enricher = new constructor(sdk);
  return (0, import_handlers.makeSqsHandler)(async (unmarshalledRecord) => {
    await enricher.enrich(unmarshalledRecord);
  });
}
class Enricher {
  sdk;
  constructor(sdk) {
    this.sdk = sdk;
  }
  async enrich(unmarshalledRecord) {
    const { createTargetModel, updateTargetModel, unmarshallSourceModel } = this.sdk;
    (0, import_assert.assert)(
      unmarshalledRecord.dynamodb?.NewImage,
      "NewImage missing from DynamoDB Stream Event. This should never happen."
    );
    const source = unmarshallSourceModel(unmarshalledRecord.dynamodb?.NewImage);
    const previous = unmarshalledRecord.dynamodb?.OldImage && unmarshallSourceModel(unmarshalledRecord.dynamodb.OldImage);
    try {
      const target = await this.load(source, previous);
      const modelToUpdate = await this.update(source, target, previous);
      if (modelToUpdate) {
        return await updateTargetModel(modelToUpdate);
      }
    } catch (err) {
      if (err instanceof import_errors.NotFoundError) {
        const modelToCreate = await this.create(source, previous);
        if (modelToCreate) {
          return await createTargetModel(modelToCreate);
        }
      }
      throw err;
    }
  }
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  Enricher,
  makeEnricher
});
