import { assert } from "@code-like-a-carpenter/assert";
import { DataIntegrityError } from "../errors";
function identity(x) {
  return x;
}
function unmarshallRawRequiredValue(value, columnName, transform) {
  assert(
    value !== null,
    () => new DataIntegrityError(`Expected ${columnName} to be non-null`)
  );
  assert(
    typeof value !== "undefined",
    () => new DataIntegrityError(`Expected ${columnName} to be defined`)
  );
  return transform(value);
}
function unmarshallRequiredField(item, fieldName, possibleColumnNames, transform = identity) {
  for (const name of possibleColumnNames) {
    if (name in item) {
      return unmarshallRawRequiredValue(item[name], name, transform);
    }
  }
  if (fieldName === "version") {
    return void 0;
  }
  throw new DataIntegrityError(`Expected ${fieldName} to be defined`);
}
function unmarshallOptionalField(item, fieldName, possibleColumnNames, transform = identity) {
  for (const name of possibleColumnNames) {
    if (name in item) {
      return transform(item[name]);
    }
  }
  return void 0;
}
export {
  unmarshallOptionalField,
  unmarshallRequiredField
};
