import { runWithNewSpan } from "@code-like-a-carpenter/telemetry";
import { AlreadyExistsError, OptimisticLockingError } from "../../errors";
async function retry(cb) {
  const maxAttempts = 5;
  for (let i = 0; i < maxAttempts; i++) {
    try {
      await runWithNewSpan(
        "retry",
        {
          attributes: {
            "com.code-like-a-carpenter.data.attempt": i,
            "com.code-like-a-carpenter.data.total_attempts": maxAttempts
          }
        },
        cb
      );
      return;
    } catch (err) {
      if (err instanceof AlreadyExistsError || err instanceof OptimisticLockingError) {
        await new Promise(
          (resolve) => setTimeout(resolve, (i + 1) * (1 + 1) * 1e3)
        );
      } else {
        throw err;
      }
    }
  }
}
export {
  retry
};
