import { assert } from "@code-like-a-carpenter/assert";
import { NotFoundError } from "../../errors";
import { makeSqsHandler } from "../common/handlers";
function makeEnricher(constructor, sdk) {
  const enricher = new constructor(sdk);
  return makeSqsHandler(async (unmarshalledRecord) => {
    await enricher.enrich(unmarshalledRecord);
  });
}
class Enricher {
  sdk;
  constructor(sdk) {
    this.sdk = sdk;
  }
  async enrich(unmarshalledRecord) {
    const { createTargetModel, updateTargetModel, unmarshallSourceModel } = this.sdk;
    assert(
      unmarshalledRecord.dynamodb?.NewImage,
      "NewImage missing from DynamoDB Stream Event. This should never happen."
    );
    const source = unmarshallSourceModel(unmarshalledRecord.dynamodb?.NewImage);
    const previous = unmarshalledRecord.dynamodb?.OldImage && unmarshallSourceModel(unmarshalledRecord.dynamodb.OldImage);
    try {
      const target = await this.load(source, previous);
      const modelToUpdate = await this.update(source, target, previous);
      if (modelToUpdate) {
        return await updateTargetModel(modelToUpdate);
      }
    } catch (err) {
      if (err instanceof NotFoundError) {
        const modelToCreate = await this.create(source, previous);
        if (modelToCreate) {
          return await createTargetModel(modelToCreate);
        }
      }
      throw err;
    }
  }
}
export {
  Enricher,
  makeEnricher
};
