import { assert } from "@code-like-a-carpenter/assert";
import { makeSqsHandler } from "../common/handlers";
function makeMultiReducer(constructor, sdk) {
  const reducer = new constructor(sdk);
  return makeSqsHandler(async (record) => {
    await reducer.reduce(record);
  });
}
class MultiReducer {
  sdk;
  constructor(sdk) {
    this.sdk = sdk;
  }
  async reduce(unmarshalledRecord) {
    const { createTargetModel, updateTargetModel, unmarshallSourceModel } = this.sdk;
    assert(
      unmarshalledRecord.dynamodb?.NewImage,
      "NewImage missing from DynamoDB Stream Event. This should never happen."
    );
    const source = unmarshallSourceModel(unmarshalledRecord.dynamodb?.NewImage);
    const previous = unmarshalledRecord.dynamodb?.OldImage && unmarshallSourceModel(unmarshalledRecord.dynamodb.OldImage);
    const sources = await this.loadSources(source, previous);
    const target = await this.loadTargets(source, previous);
    const models = await this.createOrUpdate(source, sources, target, previous);
    const modelsToCreate = models.filter(
      (model) => !("version" in model)
    );
    const modelsToUpdate = models.filter(
      (model) => "version" in model
    );
    await Promise.all([
      Promise.all(modelsToCreate.map((model) => createTargetModel(model))),
      Promise.all(modelsToUpdate.map((model) => updateTargetModel(model)))
    ]);
  }
}
export {
  MultiReducer,
  makeMultiReducer
};
