import { assert } from "@code-like-a-carpenter/assert";
import { makeSqsHandler } from "../common/handlers";
function makeReactor(constructor, sdk) {
  const reactor = new constructor(sdk);
  return makeSqsHandler(async (record) => {
    await reactor.react(record);
  });
}
class Reactor {
  sdk;
  constructor(sdk) {
    this.sdk = sdk;
  }
  react(unmarshalledRecord) {
    const { unmarshallSourceModel } = this.sdk;
    assert(
      unmarshalledRecord.dynamodb?.NewImage,
      "NewImage missing from DynamoDB Stream Event. This should never happen."
    );
    const source = unmarshallSourceModel(unmarshalledRecord.dynamodb?.NewImage);
    const previous = unmarshalledRecord.dynamodb?.OldImage && unmarshallSourceModel(unmarshalledRecord.dynamodb.OldImage);
    return this.handle(source, previous);
  }
}
export {
  Reactor,
  makeReactor
};
