import { assert } from "@code-like-a-carpenter/assert";
import { NotFoundError } from "../../errors";
import { makeSqsHandler } from "../common/handlers";
function makeReducer(constructor, sdk) {
  const reducer = new constructor(sdk);
  return makeSqsHandler(async (record) => {
    await reducer.reduce(record);
  });
}
class Reducer {
  sdk;
  constructor(sdk) {
    this.sdk = sdk;
  }
  async reduce(unmarshalledRecord) {
    const { createTargetModel, updateTargetModel, unmarshallSourceModel } = this.sdk;
    assert(
      unmarshalledRecord.dynamodb?.NewImage,
      "NewImage missing from DynamoDB Stream Event. This should never happen."
    );
    const source = unmarshallSourceModel(unmarshalledRecord.dynamodb?.NewImage);
    const previous = unmarshalledRecord.dynamodb?.OldImage && unmarshallSourceModel(unmarshalledRecord.dynamodb.OldImage);
    const sources = await this.loadSources(source, previous);
    try {
      const target = await this.loadTarget(source, previous);
      const modelToUpdate = await this.update(
        source,
        sources,
        target,
        previous
      );
      if (modelToUpdate) {
        await updateTargetModel(modelToUpdate);
        return;
      }
    } catch (err) {
      if (err instanceof NotFoundError) {
        const modelToCreate = await this.create(source, sources, previous);
        if (modelToCreate) {
          await createTargetModel(modelToCreate);
          return;
        }
      }
      throw err;
    }
  }
}
export {
  Reducer,
  makeReducer
};
