import type { ResultType } from '../../types';
import type { Handler } from '../common/handlers';
import type { UnmarshalledDynamoDBRecord } from '../common/unmarshall-record';
interface SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT> {
    createTargetModel: (target: CREATE_TARGET_INPUT) => Promise<ResultType<TARGET>>;
    unmarshallSourceModel: (item: Record<string, unknown>) => SOURCE;
    updateTargetModel: (target: UPDATE_TARGET_INPUT) => Promise<ResultType<TARGET>>;
}
/**
 * Makes an SQS handler that expects the payload to be a DynamoDB Stream Record.
 */
export declare function makeReducer<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>(constructor: ReducerConstructor<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>, sdk: SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>): Handler;
export interface ReducerConstructor<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT> {
    new (sdk: SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>): Reducer<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>;
}
export declare abstract class Reducer<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT> {
    sdk: SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>;
    constructor(sdk: SDK<SOURCE, TARGET, CREATE_TARGET_INPUT, UPDATE_TARGET_INPUT>);
    reduce(unmarshalledRecord: UnmarshalledDynamoDBRecord): Promise<void>;
    protected abstract loadTarget(source: SOURCE, previous: SOURCE | undefined): Promise<TARGET>;
    protected abstract loadSources(source: SOURCE, previous: SOURCE | undefined): Promise<readonly SOURCE[]>;
    protected abstract create(source: SOURCE, sources: readonly SOURCE[], previous: SOURCE | undefined): Promise<CREATE_TARGET_INPUT | undefined>;
    protected abstract update(source: SOURCE, sources: readonly SOURCE[], target: TARGET, previous: SOURCE | undefined): Promise<UPDATE_TARGET_INPUT | undefined>;
}
export {};
//# sourceMappingURL=make-reducer.d.ts.map